<?php
/*
 * $Horde: chora/cvs.php,v 1.102 2001/05/24 18:23:39 avsm Exp $
 *
 * Copyright 1999, 2000, 2001 Anil Madhavapeddy <anil@recoil.org>
 * Copyright 1999, 2000, 2001 Charles Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

define('CHORA_BASE', dirname(__FILE__));
require_once CHORA_BASE . '/lib/base.php';
require_once HORDE_BASE . '/config/mime_mapping.php';
require_once HORDE_BASE . '/config/mime_drivers.php';
require_once HORDE_BASE . '/lib/Text.php';
require_once HORDE_BASE . '/lib/MIME/Magic.php';
require_once HORDE_BASE . '/lib/MIME/Viewer.php';

if ($conf['compress_pages']) {
    include_once 'HTTP/Compress.php';
    HTTP_Compress::start();
}

if (@is_dir($fullname)) {

    /* checkError is the error trapping function */
    checkError($dir = $CVS->queryDir($where));

    $atticFlags = $acts['sa'] ? CVSLIB_ATTIC_SHOW : CVSLIB_ATTIC_HIDE;

    checkError($dir->browseDir(CVSLIB_LOG_QUICK, $atticFlags));
    $dir->applySort($acts['sbt'],$acts['ord']);
    checkError($dirList = $dir->queryDirList());
    checkError($fileList = $dir->queryFileList($atticFlags));

    /* Decide what title to display */
    if ($where == '') {
        $title = $conf['options']['introTitle'];
    }
    else {
        $title = "CVS Directory of /$where";
    }

    if ($acts['sa']) {
        $extraLink='<a href="'.url('cvs',$where, array('sa'=>0)).'">Hide Deleted Files</a>';
    } else {
        $extraLink='<a href="'.url('cvs',$where, array('sa'=>1)).'">Show Deleted Files</a>';
    }

    include $conf['paths']['templates'] . '/common-header.inc';

    foreach (array('age','rev','name','author') as $u) {
        $umap = array('age' => CVSLIB_SORT_AGE, 'rev' => CVSLIB_SORT_REV,
                      'name' => CVSLIB_SORT_NAME, 'author' => CVSLIB_SORT_AUTHOR);
        $arg = array('sbt' => $umap[$u]);
        if ($acts['sbt'] == $umap[$u]) {
            $arg['ord'] = !$acts['ord']; 
        }
        $url[$u] = url('cvs', $where, $arg);
    }
 
    /* Print out the directory header */
    $printAllCols = sizeof($fileList);
    include($conf['paths']['templates'] . '/directory/header.inc');

    /* Unless we're at the top, display the 'back' bar */
    if ($where != '') {
        $url = url('cvs', preg_replace('|[^/]+$|', '', $where));
        include($conf['paths']['templates'] . '/directory/back.inc');
    }

    /* Display all the directories first */
    $dirrow = 0;
    while (list(,$currentDir) = each($dirList)) {
        $dirrow = ++$dirrow % 2;
        $url = url('cvs', "$where/$currentDir");
        $currDir = Text::htmlallspaces($currentDir);
        include($conf['paths']['templates'] . '/directory/dir.inc');
    }

    /* Display all of the files in this directory */
    while (list(,$currFile) = each($fileList)) {
        $dirrow = ++$dirrow % 2;
        $lg = $currFile->queryLastLog();
        $realname = $currFile->queryName();
        $mimeType = MIME_Magic::extToMIME( CVSLib_File::getExtension($realname) );
         
        $icon = $conf['horde']['paths']['graphics'] . '/mime/' . MIME_Viewer::getIcon($mimeType);

        $aid = $lg->queryAuthor();
        $author = showAuthorName($aid);
        $head = $currFile->queryHead();
        $date = $lg->queryDate();
        $log  = $lg->queryLog();
        $attic = $currFile->isAtticFile();
        $fileName = $where. ($attic ? '/Attic' : ''). "/$realname";
        $name = Text::htmlallspaces($realname);
        $url = url('cvs', $fileName);
        $readableDate = CVSLib_File::readableTime($date);
        if ($log) {
            $shortLog = str_replace("\n" , " - ", 
                trim(substr($log, 0, $conf['options']['shortLogLength']-2)));
            if (strlen($log) > 80) {
                $shortLog .= "...";
            }
        }
        include($conf['paths']['templates'].'/directory/file.inc');
    }
    /* Display the options control panel at the bottom */
    $formwhere = $scriptName . '/' . $where;

    include $conf['paths']['templates'] . '/directory/footer.inc';
    include $conf['paths']['templates'] . '/common-footer.inc';

} else if (@is_file($fullname . ',v')) {

    $fl = new CVSLib_File($CVS, $fullname, CVSLIB_LOG_FULL);
    checkError($fl->getBrowseInfo());

    $fl->applySort(CVSLIB_SORT_AGE);

    $title = "CVS Log for $where";

    $upwhere = preg_replace('|[^/]+$|', '', $where);

    $onb = Horde::getFormData('onb', 0);
    $isBranch = isset($onb) && isset($fl->branches[$onb]) ? $fl->branches[$onb] : '';
    $extraLink = '<a href="'. url('history', $where). '">Switch to Branch View</a>';

    include $conf['paths']['templates'] . '/common-header.inc';
    include $conf['paths']['templates'] . '/log/header.inc';

    $mimeType = MIME_Magic::extToMIME( CVSLib_File::getExtension($fullname) );  
    $defaultTextPlain = ($mimeType == 'text/plain');

    while(list(,$lg)=each($fl->logs)) {

        $rev = $lg->rev;

        /* Are we sticking only to one branch ? */
        if ($onb && CVSLib_Rev::valid($onb)) {

            /* If so, if we are on the branch itself, let it through */
            if (substr($rev,0,strlen($onb)) != $onb) {

                /* We are not on the branch, see if we are on a trunk
                 * branch below the branch */
                $baseRev = CVSLib_Rev::strip($onb, 1);

                /* Check we are at the same level of branching or less */
                if (substr_count($rev,'.') <= substr_count($baseRev,'.')) {

                   /* If we are at the same level, and the revision is
                    * less, then let the revision through, since it was
                    * committed before the branch actually took place
                    */
                   if (CVSLib_Rev::cmp($rev, $baseRev) > 0) {
                      continue;
                   }

                } else {
                  continue;
                }
            }
        }

        $textURL= url('co', $where, array('r'=>$rev));
        $commitDate = gmdate('jS F Y, g:ia', $lg->date);
        $readableDate = CVSLib_File::readableTime($lg->date, true);

        $aid = $lg->queryAuthor();
        $author = showAuthorName($aid, true);

        if (!empty($lg->tags)) {
            $commitTags = implode(', ', $lg->tags);
        } else { 
            $commitTags = '';
        }

        $branchPointsArr = array();
        foreach ($lg->querySymbolicBranches() as $symb => $bra) {
            $branchPointsArr[] = '<a href="' . url('cvs', $where, array('onb' => $bra)) . '">'. $symb . '</a>';
        }

        /* Calculate the current branch name and revision */
        $branchPoints = implode(' , ', $branchPointsArr);
        $branchRev = CVSLib_Rev::strip($rev, 1);
        if (@isset($fl->branches[$branchRev])) {
            $branchName = $fl->branches[$branchRev];
        } else {
            $branchName = '';
        }

        if ($prevRevision = CVSLib_Rev::prev($lg->queryRevision())) {
            $changedLines = $lg->queryChangedLines();
            $diffURL = url('diff', $where, array('r1'=>$prevRevision,'r2'=>$rev, 'f'=>'h'));
            $longDiffURL = url('diff', $where, array('r1'=>$prevRevision,'r2'=>$rev, 'f'=>'h', 'num'=>10));
            $uniDiffURL = url('diff', $where, array('r1'=>$prevRevision,'r2'=>$rev,'f'=>'u'));
        }

        $logMessage = htmlify($lg->log);
        include($conf['paths']['templates'].'/log/rev.inc');
    }

    $first = end($fl->logs);
    $diffValueLeft  = $first->rev;
    $diffValueRight = $fl->queryRevision();

    $sel = '';
    while (list($sm,$rv) = each($fl->symrev)) {
        $sel .= '<option value="'.$rv.'">'.$sm.'</option>';
    }

    $selAllBranches = '';
    while (list($num,$sym) = each($fl->branches)) {
        $selAllBranches .= '<option value="'.$num.'">'.$sym;
    }

    include $conf['paths']['templates'] . '/log/request.inc';
    include $conf['paths']['templates'] . '/common-footer.inc';

} else {
    fatal('404 Not Found', "$where: no such file or directory");
}

if ($conf['compress_pages']) {
    HTTP_Compress::output();
}

?>
