<?php

/*
 * $Horde: chora/diff.php,v 1.37 2001/05/23 17:22:40 chuck Exp $
 *
 * Copyright 2000, 2001 Anil Madhavapeddy <anil@recoil.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

define('CHORA_BASE', dirname(__FILE__));
require_once CHORA_BASE . '/lib/base.php';
require_once HORDE_BASE . '/lib/Text.php';

if ($conf['compress_pages']) {
    include_once 'HTTP/Compress.php';
    HTTP_Compress::start();
}

/* Spawn the repository and file objects */

$fl = new CVSLib_File($CVS, $CVS->cvsRoot() . '/' . $where);
checkError($fl->getBrowseInfo());

/* Initialise the form variables correctly.
 * If r1/r2 are empty, then use the corresponding text field instead */

$r1 = Horde::getFormData('r1', 0);
$r2 = Horde::getFormData('r2', 0);

if (!$r1) $r1 = Horde::getFormData('tr1');
if (!$r2) $r2 = Horde::getFormData('tr2');

/* If no context-size has been specified, default to 3 */

$num = Horde::getFormData('num', 3);

/* If no type has been specified, then default to unified */

$f = Horde::getFormData('f', 'h');

/* Figure out what type of diff has been requested */

switch ($f) {
case 'u':
    $type = CVSLIB_DIFF_UNIFIED;
    break;
case 'h':
    $type = CVSLIB_DIFF_UNIFIED;
    break;
case 's':
    $type = CVSLIB_DIFF_COLUMN;
    break;
case 'c':
    $type = CVSLIB_DIFF_CONTEXT;
    break;
case 'e':
    $type = CVSLIB_DIFF_ED;
    break;
default:
    $type = CVSLIB_DIFF_UNIFIED;
    break;
}

/* Ensure that we have valid revision numbers */
if (!CVSLib_Rev::valid($r1) || !CVSLib_Rev::valid($r2)) {
    checkError(new CVSLib_Error(CVSLIB_NOT_FOUND, 'Malformed Query'));
}

/* All is ok, proceed with the diff */
/* A plain-text diff */
else if ($f != 'h') {
    header("Content-Type: text/plain\n\n");
    echo implode("\n",CVSLib_Diff::get($CVS, $fl, $r1, $r2, $type, $num));

/* Human-Readable diff */
} else {
  
    /* Output standard header information for the page */
    $filename = preg_replace('/^.*\//', '', $where);
    $pathname = preg_replace('/[^\/]*$/', '', $where);
    $title = "Diff for $where between version $r1 and $r2";

    include $conf['paths']['templates'] . '/common-header.inc';
    include $conf['paths']['templates'] . '/diff/hr/header.inc';

    /* Retrieve the tree of changes from CVSLib */
    $lns = CVSLib_Diff::humanReadable(
           CVSLib_Diff::get($CVS, $fl, $r1, $r2, CVSLIB_DIFF_UNIFIED, $num));
    /* TODO: check for errors here (CVSLib_Error returned) - avsm */
    /* Is the diff empty? */
    if (!sizeof($lns)) {
        include($conf['paths']['templates'].'/diff/hr/nochange.inc');
    } else
    /* Iterate through every header block of changes */
    foreach ($lns as $header) {
        $lefthead = Text::htmlspaces(@$header['oldline']);
        $righthead = Text::htmlspaces(@$header['newline']);
        $headfunc = Text::htmlspaces(@$header['function']);
        include($conf['paths']['templates'].'/diff/hr/row.inc');

        /* Each header block consists of a number of changes (add, remove, change) */
        foreach ($header['contents'] as $change) {
            switch ($change['type']) {
              case CVSLIB_DIFF_ADD:
                  foreach ($change['lines'] as $line) {
                      $line = Text::htmlspaces($line);
                      include($conf['paths']['templates'].'/diff/hr/add.inc');
                  }
                  break;
              case CVSLIB_DIFF_REMOVE:
                  foreach ($change['lines'] as $line) {
                      $line = Text::htmlspaces($line);
                      include($conf['paths']['templates'].'/diff/hr/remove.inc');
                  }
                  break;
              case CVSLIB_DIFF_EMPTY:
                  $line = Text::htmlspaces($change['line']);
                  include($conf['paths']['templates'].'/diff/hr/empty.inc');
                  break;
              case CVSLIB_DIFF_CHANGE:
                  /* Pop the old/new stacks one by one, until both are empty */
                  while (sizeof($change['old']) || sizeof($change['new'])) {
                      if ($left = array_shift($change['old'])) {
                          $left = Text::htmlspaces($left);
                      }
                      if ($right = array_shift($change['new'])) {
                          $right = Text::htmlspaces($right);
                      }
                      include($conf['paths']['templates'].'/diff/hr/change.inc');
                  }
                  break;
            }
        }
    }

    /* print legend */
    include $conf['paths']['templates'] . '/diff/hr/footer.inc';
    include $conf['paths']['templates'] . '/common-footer.inc';
}


if ($conf['compress_pages']) {
    HTTP_Compress::output();
}

?>
